'''
Created on April 2011 for Pyweek
@author: Martin Villalba (aka akira44)
'''

import pygame
import Stuff
import Tasks

class Coworkers(pygame.sprite.Sprite):
    """Abstract class, for every kind of coworker.
    Note that a fax machine, for instance, is also a coworker"""
    STILL = 0
    WORKING = 1
    BROKEN = 2
    
    def __init__(self):
        raise NotImplementedError("Not implemented")
        
    def set_status(self, x, y):
        self.pos = [x,y]
        self.rect.x = x * 60
        self.rect.bottom = (y+1) * 60
        self.work_counter = 0 # This should be unnecesary
        self.is_broken=False
        self.action = self.STILL
        
    def work_dammit(self):
        ret = Tasks.Tasks.NONE
        if(not self.is_broken):
            if(self.work_counter == 0):
                self.work_counter += 1
                ret = self.mytask
                self.action = self.WORKING
        return ret
    
    def update(self):
        """Updated the image and status of a coworker"""
        if(self.work_counter != 0): self.work_counter = (self.work_counter + 1) % 60
        self.framecounter += 1
        if(self.framecounter == 15):
            self.framecounter = 0
            curr_act = 0
            if(self.action == self.STILL): curr_act = 0
            elif(self.action == self.WORKING): curr_act = 1
            elif(self.action == self.BROKEN): curr_act = 2
        
            baseframe = 0
            for i in range(0, curr_act):
                baseframe += self.numframes[i]
        
            if(self.action == self.WORKING and self.frame == self.numframes[curr_act]-1):
                self.frame = 0
                self.action = self.STILL
                baseframe = 0
            else:
                self.frame = (self.frame + 1) % self.numframes[curr_act]
            self.image = self.frames[baseframe + self.frame]
            #self.rect.x = self.pos[0] * 60
            #self.rect.bottom = ((self.pos[1]+1) * 60)

    
    def change_broken_status(self, status):
        """Breaks (or fixes) a certain coworker"""
        if(status):
            self.action = self.BROKEN
        else:
            self.action = self.STILL
        self.frame = 0
        self.is_broken = status
    
    def get_broken_status(self):
        """Returns the status of a coworker"""
        return self.is_broken
    
    def load_frames(self, path):
        """Loads the animation frames for a coworker"""
        file = path + '/config.txt'
        f = open(file, 'r')
        buffer = f.readline()
        while(buffer[0]=='#'): buffer = f.readline()
        
        numframes = buffer.split(' ')
        for i in range(0,len(numframes)): numframes[i]=int(numframes[i])
        f.close()
        
        frames = []
        # 3 actions: still, working and broken
        for i in range(0,3):
            if (i==0): action = 'still'
            if (i==1): action = 'working'
            if (i==2): action = 'broken'

            for j in range(0, numframes[i]):
                file = path + '/{0}_{1}.png'.format(action, j)
                frames += [Stuff.load_image(file, -1)[0]]
        
        # The image is just random, but I need this rect
        temp_rect = Stuff.load_image(path + '/still_0.png', -1)[1]
        return (frames,numframes,temp_rect)
    
    def get_pos(self):
        return self.pos
    
    def play_sound(self):
        self.sound.play()