'''
Created on April 2011 for Pyweek
@author: Martin Villalba (aka akira44)
'''

import pygame
import Stuff
import Wall
import VWall
import Watercooler
import Fax
import Fix
import Telephone
import Photocopier
import Typewriter
import Coffee
import Lamp
import Plant
import Hero
import Todo
import Caffeine
import Globals
import Tasks
import Workday
import os

class office():
    """The representation of the whole office"""

    def __init__(self):
        # I'm not sure if I'll even need this     
        self.tilesize = [60,60]
        dir = os.path.abspath("data/floor.jpg")
        self.background, self.bg_rect = Stuff.load_image(dir)
        dir = os.path.abspath("data/carpet.jpg")
        self.carpet, self.carpet_rect = Stuff.load_image(dir)
        self.todo_list = Todo.Todo()
        self.caffeine = Caffeine.Caffeine()
        self.clock = Workday.Workday()
        self.player = None
        self.carpet_map = [[0 for i in range(0,8)] for j in range(0,11)]
        
    def start(self, level_num):
        level_file = os.path.abspath('data/levels/' + str(level_num) + '.txt')
        f = open(level_file, 'r')

        # First of all, lets clean the map
        for i in range(0,8):
            for j in range(0,11):
                Globals.map[j][i] = None
        # Should I acquire a lock?
        allowable = []
        for i in range(0,8):
            buffer = f.readline()
            for j in range(0,11):
                if(buffer[j]=='.'):
                    pass
                elif(buffer[j]==','):
                    self.carpet_map[j][i]=1
                elif(buffer[j]=='x'):
                    Globals.map[j][i]=Hero.hero()
                    self.player = Globals.map[j][i] 
                    Globals.map[j][i].start(j, i)
                elif(buffer[j]=='0'):
                    Globals.map[j][i]=Wall.Wall()
                    Globals.map[j][i].set_status(j, i)
                elif(buffer[j]=='1'):
                    Globals.map[j][i]=VWall.VWall()
                    Globals.map[j][i].set_status(j, i)                    
                elif(buffer[j]=='2'):
                    Globals.map[j][i]=Watercooler.Watercooler()
                    Globals.map[j][i].set_status(j, i)
                    if (Tasks.Tasks.WATER not in allowable):
                        allowable.append(Tasks.Tasks.WATER)
                elif(buffer[j]=='3'):
                    Globals.map[j][i]=Fax.Fax()
                    Globals.map[j][i].set_status(j, i)
                    if (Tasks.Tasks.FAX not in allowable):
                        allowable.append(Tasks.Tasks.FAX)
                elif(buffer[j]=='4'):
                    Globals.map[j][i]=Coffee.Coffee()
                    Globals.map[j][i].set_status(j, i)
                elif(buffer[j]=='5'):
                    Globals.map[j][i]=Telephone.Telephone()
                    Globals.map[j][i].set_status(j, i)
                    if (Tasks.Tasks.TELEPHONE not in allowable):
                        allowable.append(Tasks.Tasks.TELEPHONE)
                elif(buffer[j]=='6'):
                    Globals.map[j][i]=Photocopier.Photocopier()
                    Globals.map[j][i].set_status(j, i)
                    if (Tasks.Tasks.PHOTOCOPIER not in allowable):
                        allowable.append(Tasks.Tasks.PHOTOCOPIER)
                elif(buffer[j]=='7'):
                    Globals.map[j][i]=Typewriter.Typewriter()
                    Globals.map[j][i].set_status(j, i)
                    if (Tasks.Tasks.TYPEWRITER not in allowable):
                        allowable.append(Tasks.Tasks.TYPEWRITER)
                elif(buffer[j]=='8'):
                    Globals.map[j][i]=Fix.Fix()
                    Globals.map[j][i].set_status(j, i, self.todo_list)                    
                elif(buffer[j]=='a'):
                    Globals.map[j][i]=Lamp.Lamp()
                    Globals.map[j][i].set_status(j, i)                    
                elif(buffer[j]=='b'):
                    Globals.map[j][i]=Plant.Plant()
                    Globals.map[j][i].set_status(j, i)

        f.close()
        # Other initializations
        self.todo_list.start(5+(2*level_num), level_num, allowable)
        self.caffeine.start(100)
        self.clock.start(3600 + 120*level_num)
    
    def update(self):
        # Let's decrease the time
        self.clock.pass_time()
        
        screen = pygame.display.get_surface()
        mysprites = []
        
        # Let's draw the floor now,
        # and while we are at it, let's update everything
        for i in range(0, Globals.mapsize[0]):
            for j in range(Globals.statusbar[1][1], Globals.mapsize[1]):
                self.bg_rect.left = self.tilesize[0] * i
                self.bg_rect.top = self.tilesize[1] * j
                if(self.carpet_map[i][j]==1):
                    screen.blit(self.carpet,self.bg_rect)
                else:
                    screen.blit(self.background,self.bg_rect)
                if (Globals.map[i][j] is not None):
                    mysprites += [Globals.map[i][j]]
                    Globals.map[i][j].update()
                    
                    # If there was some work done by a coworker,
                    # pass it up to the work manager
                    if(isinstance(Globals.map[i][j],Hero.hero)):
                        last_work, last_worker = Globals.map[i][j].worked()
                        if(last_work != Tasks.Tasks.NONE):
                            self.__make_work(last_work, last_worker)
                    
        # TO-DO: a fast way of doing this
        # (where "this" is rendering from back to front)
        mysprites = sorted(mysprites, key=lambda x:x.rect.bottom)
        
        # And now, we draw the coworkers
        for i in mysprites:
            screen.blit(i.image,i.rect)
            
        # Finally, the board, clock, and other decorative elements
        self.caffeine.set_level(self.player.get_coffee())
        self.caffeine.update()
        screen.blit(self.todo_list.image, self.todo_list.rect)
        screen.blit(self.caffeine.image, self.caffeine.rect)
        screen.blit(self.clock.image, self.clock.rect)
            
    def im_done(self):
        """ The workday has ended"""
        return self.clock.seconds_left <= 0 or self.todo_list.im_done() or not self.player.is_awake()
    
    def i_still_have_a_job(self):
        """ Did I achieve the targets? """
        return self.todo_list.im_done() and self.player.is_awake() and self.clock.seconds_left > 0
    
    def i_fell_asleep(self):
        return not self.player.is_awake()
    
    def __make_work(self, task, coworker):
        """ Notes the work done by the object at the (x,y) position"""
        if(task == Tasks.Tasks.COFFEE):
            # Increase the coffee level
            self.player.set_coffee(+10)

        self.todo_list.accomplish(task, coworker)
        #elif(self.todo_list.accomplish(task, coworker)):
            #Check if we won
            
            #if(self.todo_list.im_done()):
                # End the level
                #pass
                #print "You won!"
                