'''
Created on April 2011 for Pyweek
@author: Martin Villalba (aka akira44)
'''

import pygame
import Stuff
import Globals
import Tasks
import os
#import copy
#import Hero
from collections import deque

class Repairman(pygame.sprite.Sprite):
    """Class for the repairing guy"""

    def __init__(self):
        pygame.sprite.Sprite.__init__(self)

        self.pos = [-1,-1]
        self.machine = None
        self.todo = None
        
        # Load the animation files        
        file = os.path.abspath('data/repair/config.txt')
        f = open(file, 'r')
        buffer = f.readline()
        while(buffer[0]=='#'): buffer = f.readline()
        
        self.numframes = buffer.split(' ')
        for i in range(0,len(self.numframes)): self.numframes[i]=int(self.numframes[i])
        f.close()
        
        self.frames = []
        # 7 actions so far
        for i in range(0,8):
            if (i==0): action, direction = 'walk','l'
            if (i==1): action, direction = 'walk','r'
            if (i==2): action, direction = 'walk','u'
            if (i==3): action, direction = 'walk','d'
            if (i==4): action, direction = 'still','l'
            if (i==5): action, direction = 'still','r'
            if (i==6): action, direction = 'still','u'
            if (i==7): action, direction = 'still','d'

            for j in range(0, self.numframes[i]):
                file = os.path.abspath('data/repair/{0}_{1}_{2}.png'.format(action, direction, j))
                self.frames += [Stuff.load_image(file, -1)[0]]
        
        # The image is just random, but I need this rect
        file = os.path.abspath('data/repair/still_r_0.png')
        self.image, self.rect = Stuff.load_image(file, -1)

    def start(self, x, y, machine, todo):
        
        # Is he in between squares?
        self.ismoving= False        
        # Current and previous direction
        self.direction = self.prev_direction = [0,0]          
        # Current animation frame
        self.curr_frame = 0
        # Timer for image switching
        self.frametimer = 0
        
        self.pos = [x,y]
        self.work_done = False
        self.machine = machine
        self.todo = todo

        # Lets find a suitable destination


        dx, dy = self.machine.get_pos()[0], self.machine.get_pos()[1]
        if(Globals.could_move(dx+1,dy)):
            self.destination = [dx+1, dy]
        elif(Globals.could_move(dx-1,dy)):
            self.destination = [dx-1, dy]
        elif(Globals.could_move(dx,dy+1)):
            self.destination = [dx, dy+1]
        elif(Globals.could_move(dx,dy-1)):
            self.destination = [dx, dy-1]
        else:
            # This is a problem when the player is blocking the only way out.
            # Let's try and find which position is likely to get cleared
            print "This shouldn't have happened, sorry!"
        
    def stop(self):
        # Hides the player until next time
        Globals.map[self.pos[0]][self.pos[1]] = None
        self.pos = [-1,-1]
        

    def update(self):
        """Moves the player in any direction"""
             
        if(self.pos == [-1,-1]): return
        
        if(self.ismoving):
            # TO-DO: si hace click en shift, va mas rapido
            # (pero se cansa mas) 
            self.rect.x += self.direction[0]*2
            self.rect.bottom += self.direction[1]*2
            
            # Bug fix - I don't know why this works, but it does
            self.prev_direction = self.direction
            
            if self.rect.x % 60 == 0 and self.rect.bottom % 60 == 0:
                self.ismoving = False
                self.pos = [self.rect.x / 60, (self.rect.bottom / 60)-1]
        else:
            self.ismoving = True
            # Here comes the path-searching algorithm
            v = self.__find_path()
            
            # V me dice hacia donde moverme, asi que le voy a hacer caso
            mov_x, mov_y = 0,0
            if(v[0]==1): mov_x, mov_y = 1, 0
            elif(v[0] == -1): mov_x, mov_y = -1, 0
            elif(v[1] == 1): mov_x, mov_y = 0, 1
            elif(v[1] == -1): mov_x, mov_y = 0, -1
                
            self.prev_direction = self.direction
            Globals.cand.acquire()
            if (Globals.can_move(self.pos[0]+mov_x,self.pos[1]+mov_y)):
                # I can move to the next square, so I do    
                self.direction = [mov_x,mov_y]
                # Even I want to kick myself for doing this
                Globals.map[self.pos[0]+mov_x][self.pos[1]+mov_y]=self
                Globals.map[self.pos[0]][self.pos[1]]=None
            else:
                # There is something in the way, so I'll make it work,
                # or I reached the end of the world 
                self.ismoving = False
                if(self.direction != [0,0]):
                    if(self.pos[0]+mov_x >= 0 and self.pos[0]+mov_x <11 and \
                       self.pos[1]+mov_y>Globals.statusbar[1][1] and self.pos[1]+mov_y<8):
                        
                        # I'm stuck against something - let's see what should I do
                        if([self.pos[0]+mov_x,self.pos[1]+mov_y] == self.destination):
                            # I reached my destination
                            # Time to go back home?
                            if(self.work_done):
                                #print "Choque contra: {0}".format(Globals.map[self.pos[0]+mov_x][self.pos[1]+mov_y])
                                self.stop()
                            else:
                                #print "Alcance el objetivo"
                                #print Globals.map[self.pos[0]+mov_x][self.pos[1]+mov_y]
                                self.work_done = True
                                self.machine.change_broken_status(False)
                                self.todo.accomplish(Tasks.Tasks.FIX_THIS,None)
                                self.destination = [0,1]
                                self.direction = [0,0]
                                                        
                        else:
                            # That pesky player is in the way - let's wait for him
                            # to move out of the way by standing still
                            mov_x=mov_y=0
                            # To-Do: make the repairman stand still
                else:
                    pass
            Globals.cand.release()

        # Now, we update the animation
        self.__update_anim()
    
    def __update_anim(self):
        if(self.pos != [-1,-1]):
            # First, let's see which way we are looking
            if(self.direction[0] == 1): action = 1
            elif(self.direction[0] == -1): action = 0
            elif(self.direction[1] == 1): action = 3
            elif(self.direction[1] == -1): action = 2
            else:
                if(self.prev_direction[0] == 1): action=5
                elif(self.prev_direction[0] == -1): action=4
                elif(self.prev_direction[1] == 1): action=7
                elif(self.prev_direction[1] == -1): action=6
                else: action=7 # BUG! I should prevent this from happening
        
            # Now, lets define the next frame
            baseframe=0
            for i in range(0,action):
                baseframe+=self.numframes[i]
    
            if(self.prev_direction == self.direction):
                # Let's keep animating
                self.frametimer += 1
                if(self.frametimer == 5):
                    self.frametimer = 0
                    self.curr_frame = (self.curr_frame + 1) % self.numframes[action]
            else:
                # Direction change, let's restart the frame timer
                self.frametimer = 0
                self.curr_frame = 0

            self.image = self.frames[baseframe + self.curr_frame]
        else:
            self.image = pygame.Surface((1,1))
        
        screen = pygame.display.get_surface()
        screen.blit(self.image, self.rect)

    def is_done(self):
        return self.pos == [-1,-1]
    
    def work_dammit(self):
        return Tasks.Tasks.NONE

    def __find_path(self):
        # Lets create now some structures
        q = deque()
        mapa=[[0 for i in range(0,Globals.mapsize[1])] for i in range(0,Globals.mapsize[0])]
        Globals.cand.acquire()
        for i in range(1, Globals.mapsize[1]):
            for j in range(0, Globals.mapsize[0]):
                if(Globals.map[j][i] != None):
                    mapa[j][i]=1
        Globals.cand.release()
        
        arbol = {(self.pos[0],self.pos[1]):(self.pos[0],self.pos[1])}
        # I start with the root node, that is, my current position
        q.appendleft([self.pos[0],self.pos[1]])
        mapa[self.pos[0]][self.pos[1]]=1
        # I want to be able to reach this point
        encontrado = False
        while(len(q)>0 and not encontrado):
            v = q.pop()
            if(v[0] == self.destination[0] and v[1]== self.destination[1]):
                # Encontre el nodo
                b = (v[0],v[1])
                while(arbol[b] != (self.pos[0],self.pos[1])):
                    b=arbol[b]
                v[0]=b[0]-self.pos[0]
                v[1]=b[1]-self.pos[1]
                encontrado = True
            elif(v[0]<11 and v[0]>=0 and v[1]<8 and v[1]>0):
                # Sigo buscando el camino
                if(v[0]<10):
                    if(mapa[v[0]+1][v[1]] != 1):
                        mapa[v[0]+1][v[1]] = 1
                        q.appendleft([v[0]+1,v[1]])
                        arbol[(v[0]+1,v[1])] = (v[0],v[1])
                if(v[0]>0):
                    if(mapa[v[0]-1][v[1]] != 1):
                        mapa[v[0]-1][v[1]] = 1
                        q.appendleft([v[0]-1,v[1]])
                        arbol[(v[0]-1,v[1])] = (v[0],v[1])
                if(v[1]<7):
                    if(mapa[v[0]][v[1]+1] != 1):
                        mapa[v[0]][v[1]+1] = 1
                        q.appendleft([v[0],v[1]+1])
                        arbol[(v[0],v[1]+1)] = (v[0],v[1])
                if(v[1]>1):
                    if(mapa[v[0]][v[1]-1] != 1):
                        mapa[v[0]][v[1]-1] = 1
                        q.appendleft([v[0],v[1]-1])
                        arbol[(v[0],v[1]-1)] = (v[0],v[1])

        if(not encontrado):
            v=[0,0]
        return v
